## ecr.ui.helper.PathCompleter

src/ecr/ui/helper/PathCompleter.py
    1
    2
    3
    4
    5
    6
    7
    8
    9
    10
    11
    12
    13
    14
    15
    16
    17
    18
    19
    20
    21
    22
    23
    24
    25
    26
    27
    28
    29
    30
    31
    32
    33
    34
    35
    36
    37
    38
    39
    40
    41
    42
    43
    44
    45
    46
    47
    48
    49
    50
    51
    52
    53
    54
    55
    56
    57
    58
    59
    60
    61
    62
    63
    64
    65
    66
    67
    68
    69
    70
    71
    72
    73
    74
    75
    76
    77
    78
    79
    80
    81
    82
    83
    84
    85
    86
    87
    88
    import os
    from prompt_toolkit.completion import Completer, Completion

    class PathCompleter(Completer):
        """
        Complete for Path variables.

        :param get_paths: Callable which returns a list of directories to look into
                          when the user enters a relative path.
        :param file_filter: Callable which takes a filename and returns whether
                            this file should show up in the completion. ``None``
                            when no filtering has to be done.
        :param min_input_len: Don't do autocompletion when the input string is shorter.
        """

        def __init__(self, only_directories=False, get_paths=None, file_filter=None,
                     min_input_len=0, expanduser=False):
            assert get_paths is None or callable(get_paths)
            assert file_filter is None or callable(file_filter)
            assert isinstance(min_input_len, int)
            assert isinstance(expanduser, bool)

            self.only_directories = only_directories
            self.get_paths = get_paths or (lambda: ['.'])
            self.file_filter = file_filter or (lambda _: True)
            self.min_input_len = min_input_len
            self.expanduser = expanduser

        def get_completions(self, document, complete_event):
            text = document.text_before_cursor.split()
            if not text:
                return
            text = text[-1]

            # Complete only when we have at least the minimal input length,
            # otherwise, we can too many results and autocompletion will become too
            # heavy.
            if len(text) < self.min_input_len:
                return

            try:
                # Do tilde expansion.
                if self.expanduser:
                    text = os.path.expanduser(text)

                # Directories where to look.
                dirname = os.path.dirname(text)
                if dirname:
                    directories = [os.path.dirname(os.path.join(p, text))
                                   for p in self.get_paths()]
                else:
                    directories = self.get_paths()

                # Start of current file.
                prefix = os.path.basename(text)

                # Get all filenames.
                filenames = []
                for directory in directories:
                    # Look for matches in this directory.
                    if os.path.isdir(directory):
                        for filename in os.listdir(directory):
                            if filename.startswith(prefix):
                                filenames.append((directory, filename))

                # Sort
                filenames = sorted(filenames, key=lambda k: k[1])

                # Yield them.
                for directory, filename in filenames:
                    completion = filename[len(prefix):]
                    full_name = os.path.join(directory, filename)

                    if os.path.isdir(full_name):
                        # For directories, add a slash to the filename.
                        # (We don't add them to the `completion`. Users can type it
                        # to trigger the autocompletion themselves.)
                        filename += '/'
                    elif self.only_directories:
                        continue

                    if not self.file_filter(full_name):
                        continue

                    yield Completion(completion, 0, display=filename)
            except OSError:
                pass